'use strict';
const Generator = require('yeoman-generator');
const chalk = require('chalk');
const yosay = require('yosay');

module.exports = class extends Generator {
  prompting() {
    // Have Yeoman greet the user.
    this.log(
      yosay(`Bienvenido al generador ${chalk.red('generator-node-typescript-crud')}!`)
    );

    const prompts = [{
      type: 'list',
      name: 'type',
      message: 'Selecciona el tipo de API que quieres crear',
      choices: [
        'MongoDB CRUD',
        'Networking'
      ]
    },
    {
        type: 'input',
        name: 'name',
        message: 'Ingresa el nombre del CRUD que vas a crear...',
        default: 'resource'
      }];

    return this.prompt(prompts).then(props => {
      // To access props later use this.props.someAnswer;
      this.props = props;
    });
  }

  writing() {
    console.log('prompts', this.props);
    switch (this.props.type) {
      case 'MongoDB CRUD':
        this.fs.copyTpl(
          this.templatePath('resource/resource.controller.ts'),
          this.destinationPath(`${this.props.name}/${this.props.name}.controller.ts`),
          {
            name1: `${this.props.name.charAt(0).toUpperCase()}${this.props.name.slice(1)}`,
            name2: this.props.name
          }
        );
        this.fs.copyTpl(
          this.templatePath('resource/resource.model.ts'),
          this.destinationPath(`${this.props.name}/${this.props.name}.model.ts`),
          {
            name1: `${this.props.name.charAt(0).toUpperCase()}${this.props.name.slice(1)}`,
            name2: this.props.name
          }
        );
        this.fs.copyTpl(
          this.templatePath('resource/resource.router.ts'),
          this.destinationPath(`${this.props.name}/${this.props.name}.router.ts`),
          {
            name1: `${this.props.name.charAt(0).toUpperCase()}${this.props.name.slice(1)}`,
            name2: this.props.name
          }
        );
        this.fs.copyTpl(
          this.templatePath('resource/resource.spec.ts'),
          this.destinationPath(`${this.props.name}/${this.props.name}.spec.ts`),
          {
            name1: `${this.props.name.charAt(0).toUpperCase()}${this.props.name.slice(1)}`,
            name2: this.props.name
          }
        );
        break;
      case 'Networking':
        this.fs.copyTpl(
          this.templatePath('networking/networking.controller.ts'),
          this.destinationPath(`${this.props.name}/${this.props.name}.controller.ts`),
          {
            name1: `${this.props.name.charAt(0).toUpperCase()}${this.props.name.slice(1)}`,
            name2: this.props.name
          }
        );
        this.fs.copyTpl(
          this.templatePath('networking/networking.service.ts'),
          this.destinationPath(`${this.props.name}/${this.props.name}.service.ts`),
          {
            name1: `${this.props.name.charAt(0).toUpperCase()}${this.props.name.slice(1)}`,
            name2: this.props.name
          }
        );
        this.fs.copyTpl(
          this.templatePath('networking/networking.router.ts'),
          this.destinationPath(`${this.props.name}/${this.props.name}.router.ts`),
          {
            name1: `${this.props.name.charAt(0).toUpperCase()}${this.props.name.slice(1)}`,
            name2: this.props.name
          }
        );
        this.fs.copyTpl(
          this.templatePath('networking/networking.spec.ts'),
          this.destinationPath(`${this.props.name}/${this.props.name}.spec.ts`),
          {
            name1: `${this.props.name.charAt(0).toUpperCase()}${this.props.name.slice(1)}`,
            name2: this.props.name
          }
        );
        break;
      default:
        break;
    }

  }

  install() {
    // this.installDependencies();
  }
};
