/**
 * Using Rails-like standard naming convention for endpoints.
 * GET     /api/<%= name2 %>s              ->  index
 * POST    /api/<%= name2 %>s              ->  create
 * GET     /api/<%= name2 %>s/:id          ->  show
 * PUT     /api/<%= name2 %>s/:id          ->  upsert
 * PATCH   /api/<%= name2 %>s/:id          ->  patch
 * DELETE  /api/<%= name2 %>s/:id          ->  destroy
 */

'use strict';

import jsonpatch from 'fast-json-patch';
import <%= name1 %> from './<%= name2 %>.model';

function respondWithResult(res, statusCode) {
  statusCode = statusCode || 200;
  return function(entity) {
    if(entity) {
      return res.status(statusCode).json(entity);
    }
    return null;
  };
}

function patchUpdates(patches) {
  return function(entity) {
    try {
      // eslint-disable-next-line prefer-reflect
      jsonpatch.apply(entity, patches, /*validate*/ true);
    } catch(err) {
      return Promise.reject(err);
    }

    return entity.save();
  };
}

function removeEntity(res) {
  return function(entity) {
    if(entity) {
      return entity.remove()
        .then(() => {
          res.status(204).end();
        });
    }
  };
}

function handleEntityNotFound(res) {
  return function(entity) {
    if(!entity) {
      res.status(404).end();
      return null;
    }
    return entity;
  };
}

function handleError(res, statusCode) {
  statusCode = statusCode || 500;
  return function(err) {
    res.status(statusCode).send(err);
  };
}

// Gets a list of <%= name1 %>s
export function index(req, res) {
  return <%= name1 %>.find().exec()
    .then(respondWithResult(res))
    .catch(handleError(res));
}

// Gets a single <%= name1 %> from the DB
export function show(req, res) {
  return <%= name1 %>.findById(req.params.id).exec()
    .then(handleEntityNotFound(res))
    .then(respondWithResult(res))
    .catch(handleError(res));
}

// Creates a new <%= name1 %> in the DB
export function create(req, res) {
  return <%= name1 %>.create(req.body)
    .then(respondWithResult(res, 201))
    .catch(handleError(res));
}

// Upserts the given <%= name1 %> in the DB at the specified ID
export function upsert(req, res) {
  if(req.body._id) {
    Reflect.deleteProperty(req.body, '_id');
  }
  return <%= name1 %>.findOneAndUpdate({_id: req.params.id}, req.body, {new: true, upsert: true, setDefaultsOnInsert: true, runValidators: true}).exec()

    .then(respondWithResult(res))
    .catch(handleError(res));
}

// Updates an existing <%= name1 %> in the DB
export function patch(req, res) {
  if(req.body._id) {
    Reflect.deleteProperty(req.body, '_id');
  }
  return <%= name1 %>.findById(req.params.id).exec()
    .then(handleEntityNotFound(res))
    .then(patchUpdates(req.body))
    .then(respondWithResult(res))
    .catch(handleError(res));
}

// Deletes a <%= name1 %> from the DB
export function destroy(req, res) {
  return <%= name1 %>.findById(req.params.id).exec()
    .then(handleEntityNotFound(res))
    .then(removeEntity(res))
    .catch(handleError(res));
}
